@@ -26,7 +26,9 @@ from typing import (
     Union,
 )
 
+import numba as nb
 import numpy as np
+from numba.extending import overload, register_jitable
 from numpy.typing import DTypeLike
 
 from ..grids.base import DimensionError, DomainError, GridBase, discretize_interval
@@ -35,7 +37,7 @@ from ..grids.cartesian import CartesianGrid
 from ..tools.cache import cached_method
 from ..tools.docstrings import fill_in_docstring
 from ..tools.misc import Number, number_array
-from ..tools.numba import jit, make_array_constructor
+from ..tools.numba import get_common_numba_dtype, jit, make_array_constructor
 from ..tools.plotting import (
     PlotReference,
     napari_add_layers,
@@ -1026,8 +1028,9 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
                 \delta(\boldsymbol k - \boldsymbol k’)
 
         in spectral space, where :math:`\boldsymbol k` is the wave vector. The special
-        case :math:`\nu = 0` corresponds to white noise. Note that the components of
-        vector or tensor fields are uncorrelated.
+        case :math:`\nu = 0` corresponds to white noise. Note that the spatial
+        correlations always assume periodic boundary conditions (even if the underlying
+        grid does not) and that the components of tensor fields are uncorrelated.
 
         Args:
             grid (:class:`~pde.grids.base.GridBase`):
@@ -1521,7 +1524,7 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             raise AssertionError("Rank must be non-negative")
 
     @fill_in_docstring
-    def _apply_operator(
+    def apply_operator(
         self,
         operator: str,
         bc: Optional[BoundariesData],
@@ -1531,23 +1534,25 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         args: Optional[Dict[str, Any]] = None,
         **kwargs,
     ) -> DataFieldBase:
-        r"""apply an operator and return result as a field
+        r"""apply a (differential) operator and return result as a field
 
         Args:
             operator (str):
-                An identifier determining the registered on the grid.
+                An identifier determining the operator. Note that not all grids support
+                the same operators.
             bc:
-                The boundary conditions applied to the field.
+                Boundary conditions applied to the field before applying the operator.
                 {ARG_BOUNDARIES_OPTIONAL}
-            out (ScalarField, optional):
-                Optional scalar field to which the  result is written.
+            out (:class:`DataFieldBase`, optional):
+                Optional field to which the  result is written.
             label (str, optional):
                 Name of the returned field
             **kwargs:
                 Additional arguments affecting how the operator behaves.
 
         Returns:
-            Field with new data. This is stored at `out` if given.
+            Field data after applying the operator. This field is identical to `out` if
+            this argument was specified.
         """
         # get information about the operator
         operator_info = self.grid._get_operator_info(operator)
@@ -1571,6 +1576,210 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
 
         return out
 
+    @fill_in_docstring
+    def _apply_operator(
+        self,
+        operator: str,
+        bc: Optional[BoundariesData],
+        out: Optional[DataFieldBase] = None,
+        *,
+        label: Optional[str] = None,
+        args: Optional[Dict[str, Any]] = None,
+        **kwargs,
+    ) -> DataFieldBase:
+        r"""apply a (differential) operator and return result as a field
+
+        Args:
+            operator (str):
+                An identifier determining the operator. Note that not all grids support
+                the same operators.
+            bc:
+                Boundary conditions applied to the field before applying the operator.
+                {ARG_BOUNDARIES_OPTIONAL}
+            out (:class:`DataFieldBase`, optional):
+                Optional field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
+            **kwargs:
+                Additional arguments affecting how the operator behaves.
+
+        Returns:
+            Field data after applying the operator. This field is identical to `out` if
+            this argument was specified.
+        """
+        # deprecated on 2023-02-23
+        warnings.warn(
+            "`apply_operator` is deprecated. Use `apply_operator` instead",
+            DeprecationWarning,
+        )
+        return self.apply_operator(
+            operator, bc, out=out, label=label, args=args, **kwargs
+        )
+
+    def make_dot_operator(
+        self, backend: str = "numba", *, conjugate: bool = True
+    ) -> Callable[[np.ndarray, np.ndarray, Optional[np.ndarray]], np.ndarray]:
+        """return operator calculating the dot product between two fields
+
+        This supports both products between two vectors as well as products
+        between a vector and a tensor.
+
+        Args:
+            backend (str):
+                Can be `numba` or `numpy`, deciding how the function is constructed
+            conjugate (bool):
+                Whether to use the complex conjugate for the second operand
+
+        Returns:
+            function that takes two instance of :class:`~numpy.ndarray`, which contain
+            the discretized data of the two operands. An optional third argument can
+            specify the output array to which the result is written.
+        """
+        dim = self.grid.dim
+        num_axes = self.grid.num_axes
+
+        @register_jitable
+        def maybe_conj(arr: np.ndarray) -> np.ndarray:
+            """helper function implementing optional conjugation"""
+            return arr.conjugate() if conjugate else arr
+
+        def dot(
+            a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+        ) -> np.ndarray:
+            """numpy implementation to calculate dot product between two fields"""
+            rank_a = a.ndim - num_axes
+            rank_b = b.ndim - num_axes
+            if rank_a < 1 or rank_b < 1:
+                raise TypeError("Fields in dot product must have rank >= 1")
+            assert a.shape[rank_a:] == b.shape[rank_b:]
+
+            if rank_a == 1 and rank_b == 1:  # result is scalar field
+                return np.einsum("i...,i...->...", a, maybe_conj(b), out=out)
+
+            elif rank_a == 2 and rank_b == 1:  # result is vector field
+                return np.einsum("ij...,j...->i...", a, maybe_conj(b), out=out)
+
+            elif rank_a == 1 and rank_b == 2:  # result is vector field
+                return np.einsum("i...,ij...->j...", a, maybe_conj(b), out=out)
+
+            elif rank_a == 2 and rank_b == 2:  # result is tensor-2 field
+                return np.einsum("ij...,jk...->ik...", a, maybe_conj(b), out=out)
+
+            else:
+                raise TypeError(f"Unsupported shapes ({a.shape}, {b.shape})")
+
+        if backend == "numpy":
+            # return the bare dot operator without the numba-overloaded version
+            return dot
+
+        elif backend == "numba":
+            # overload `dot` and return a compiled version
+
+            def get_rank(arr: Union[nb.types.Type, nb.types.Optional]) -> int:
+                """determine rank of field with type `arr`"""
+                arr_typ = arr.type if isinstance(arr, nb.types.Optional) else arr
+                if not isinstance(arr_typ, (np.ndarray, nb.types.Array)):
+                    raise nb.errors.TypingError(
+                        f"Dot argument must be array, not  {arr_typ.__class__}"
+                    )
+                rank = arr_typ.ndim - num_axes
+                if rank < 1:
+                    raise TypeError("Fields in dot product must have rank >= 1")
+                return rank  # type: ignore
+
+            @overload(dot, inline="always")
+            def dot_ol(
+                a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+            ) -> np.ndarray:
+                """numba implementation to calculate dot product between two fields"""
+                # get (and check) rank of the input arrays
+                rank_a = get_rank(a)
+                rank_b = get_rank(b)
+
+                if rank_a == 1 and rank_b == 1:  # result is scalar field
+
+                    @register_jitable
+                    def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> None:
+                        out[:] = a[0] * maybe_conj(b[0])
+                        for j in range(1, dim):
+                            out[:] += a[j] * maybe_conj(b[j])
+
+                elif rank_a == 2 and rank_b == 1:  # result is vector field
+
+                    @register_jitable
+                    def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> None:
+                        for i in range(dim):
+                            out[i] = a[i, 0] * maybe_conj(b[0])
+                            for j in range(1, dim):
+                                out[i] += a[i, j] * maybe_conj(b[j])
+
+                elif rank_a == 1 and rank_b == 2:  # result is vector field
+
+                    @register_jitable
+                    def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> None:
+                        for i in range(dim):
+                            out[i] = a[0] * maybe_conj(b[0, i])
+                            for j in range(1, dim):
+                                out[i] += a[j] * maybe_conj(b[j, i])
+
+                elif rank_a == 2 and rank_b == 2:  # result is tensor-2 field
+
+                    @register_jitable
+                    def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> None:
+                        for i in range(dim):
+                            for j in range(dim):
+                                out[i, j] = a[i, 0] * maybe_conj(b[0, j])
+                                for k in range(1, dim):
+                                    out[i, j] += a[i, k] * maybe_conj(b[k, j])
+
+                else:
+                    raise NotImplementedError("Inner product for these ranks")
+
+                if isinstance(out, (nb.types.NoneType, nb.types.Omitted)):
+                    # function is called without `out` -> allocate memory
+                    rank_out = rank_a + rank_b - 2
+                    a_shape = (dim,) * rank_a + self.grid.shape
+                    b_shape = (dim,) * rank_b + self.grid.shape
+                    out_shape = (dim,) * rank_out + self.grid.shape
+                    dtype = get_common_numba_dtype(a, b)
+
+                    def dot_impl(
+                        a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+                    ) -> np.ndarray:
+                        """helper function allocating output array"""
+                        assert a.shape == a_shape
+                        assert b.shape == b_shape
+                        out = np.empty(out_shape, dtype=dtype)
+                        calc(a, b, out)
+                        return out
+
+                else:
+                    # function is called with `out` argument -> reuse `out` array
+
+                    def dot_impl(
+                        a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+                    ) -> np.ndarray:
+                        """helper function without allocating output array"""
+                        assert a.shape == a_shape
+                        assert b.shape == b_shape
+                        assert out.shape == out_shape  # type: ignore
+                        calc(a, b, out)
+                        return out  # type: ignore
+
+                return dot_impl  # type: ignore
+
+            @jit
+            def dot_compiled(
+                a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+            ) -> np.ndarray:
+                """numba implementation to calculate dot product between two fields"""
+                return dot(a, b, out)
+
+            return dot_compiled  # type: ignore
+
+        else:
+            raise ValueError(f"Unsupported backend `{backend}")
+
     def smooth(
         self: TDataField,
         sigma: float = 1,
@@ -1785,13 +1994,11 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
                 The method for extracting scalars as described in
                 :meth:`DataFieldBase.to_scalar`.
             transpose (bool):
-                Determines whether the transpose of the data should is plotted
+                Determines whether the transpose of the data is plotted
             \**kwargs:
-                Additional keyword arguments that affect the image. For instance, some
-                fields support a `scalar` argument that determines how they are
-                converted to a scalar. Non-Cartesian grids might support a
-                `performance_goal` argument to influence how an image is created from
-                the raw data. Finally, the remaining arguments are are passed to
+                Additional keyword arguments that affect the image. Non-Cartesian grids
+                might support `performance_goal` to influence how an image is created
+                from raw data. Finally, remaining arguments are passed to
                 :func:`matplotlib.pyplot.imshow` to affect the appearance.
 
         Returns:
@@ -1982,11 +2189,40 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             {PLOT_ARGS}
             \**kwargs:
                 All additional keyword arguments are forwarded to the actual
-                plotting function.
+                plotting function determined by `kind`.
 
         Returns:
-            :class:`PlotReference`: Instance that contains information to update
-            the plot with new data later.
+            :class:`~pde.tools.plotting.PlotReference`: Instance that contains
+            information to update the plot with new data later.
+
+        Tip:
+            Typical additional arguments for the various plot kinds include
+
+            * :code:`kind == "line"`:
+
+              - `scalar`: sets method for extracting scalars as described in
+                :meth:`DataFieldBase.to_scalar`.
+              - `extract`: method used for extracting the line data.
+              - `ylabel`: Label of the y-axis. If omitted, the label is chosen
+                automatically from the data field.
+              - Additional arguments are passed to :func:`matplotlib.pyplot.plot`
+
+            * :code:`kind == "image"`:
+
+              - `colorbar`: Determines whether a colorbar is shown
+              - `scalar`: sets method for extracting scalars as described in
+                 :meth:`DataFieldBase.to_scalar`.
+              - `transpose` determines whether the transpose of the data is plotted
+              - Most remaining arguments are passed to :func:`matplotlib.pyplot.imshow`
+
+            * :code:`kind == `"vector"`:
+
+              - `method` can be either `quiver` or `streamplot`
+              - `transpose` determines whether the transpose of the data is plotted
+              - `max_points` sets max. number of points along each axis in quiver plots
+              - Additional arguments are passed to :func:`matplotlib.pyplot.quiver` or
+                :func:`matplotlib.pyplot.streamplot`.
+
         """
         # determine the correct kind of plotting
         if kind == "auto":
