@@ -196,114 +196,6 @@ class VectorField(DataFieldBase):
 
     __matmul__ = dot  # support python @-syntax for matrix multiplication
 
-    def make_dot_operator(
-        self, backend: str = "numba", *, conjugate: bool = True
-    ) -> Callable[[np.ndarray, np.ndarray, Optional[np.ndarray]], np.ndarray]:
-        """return operator calculating the dot product involving vector fields
-
-        This supports both products between two vectors as well as products between a
-        vector and a tensor.
-
-        Warning:
-            This function does not check types or dimensions.
-
-        Args:
-            backend (str):
-                The backend (e.g., 'numba' or 'scipy') used for this operator.
-            conjugate (bool):
-                Whether to use the complex conjugate for the second operand
-
-        Returns:
-            function that takes two instance of :class:`~numpy.ndarray`, which contain
-            the discretized data of the two operands. An optional third argument can
-            specify the output array to which the result is written. Note that the
-            returned function is jitted with numba for speed.
-        """
-        dim = self.grid.dim
-
-        if backend == "numba":
-            # create the dot product using a numba compiled function
-
-            if conjugate:
-                # create inner function calculating the dot product using conjugate
-
-                @register_jitable
-                def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> np.ndarray:
-                    """calculate dot product between fields `a` and `b`"""
-                    out[:] = a[0] * b[0].conjugate()  # overwrite potential data in out
-                    for i in range(1, dim):
-                        out[:] += a[i] * b[i].conjugate()
-                    return out
-
-            else:
-                # create the inner function calculating the dot product
-
-                @register_jitable
-                def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> np.ndarray:
-                    """calculate dot product between fields `a` and `b`"""
-                    out[:] = a[0] * b[0]  # overwrite potential data in out
-                    for i in range(1, dim):
-                        out[:] += a[i] * b[i]
-                    return out
-
-            @jit
-            def dot(
-                a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
-            ) -> np.ndarray:
-                if out is None:
-                    # we need to determine the dtype of the calculation, which we here
-                    # do by simply multiplying the first elements
-                    c = np.array(a.flat[0] * b.flat[0])
-                    out = np.empty(b.shape[1:], dtype=c.dtype)
-                return calc(a, b, out)  # type: ignore
-
-        elif backend == "numpy":
-            # create the dot product using basic numpy functions
-
-            def calc(
-                a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
-            ) -> np.ndarray:
-                """inner function doing the actual calculation of the dot product"""
-                if a.shape == b.shape:
-                    # dot product between vector and vector
-                    if out is None:
-                        # TODO: Remove this construct once we make numpy 1.20 a minimal
-                        # requirement. Earlier version of numpy do not support out=None
-                        # correctly and we thus had to use this work-around
-                        return np.einsum("i...,i...->...", a, b)  # type: ignore
-                    else:
-                        return np.einsum("i...,i...->...", a, b, out=out)
-
-                elif a.shape == b.shape[1:]:
-                    # dot product between vector and tensor
-                    if out is None:
-                        # TODO: Remove this construct once we make numpy 1.20 a minimal
-                        # requirement. Earlier version of numpy do not support out=None
-                        # correctly and we thus had to use this work-around
-                        return np.einsum("i...,ij...->j...", a, b)  # type: ignore
-                    else:
-                        return np.einsum("i...,ij...->j...", a, b, out=out)
-
-                else:
-                    raise ValueError(f"Unsupported shapes ({a.shape}, {b.shape})")
-
-            if conjugate:
-                # create inner function calculating the dot product using conjugate
-
-                def dot(
-                    a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
-                ) -> np.ndarray:
-                    """dot product with conjugated second operand"""
-                    return calc(a, b.conjugate(), out=out)  # type: ignore
-
-            else:
-                dot = calc
-
-        else:
-            raise ValueError(f"Undefined backend `{backend}")
-
-        return dot  # type: ignore
-
     def outer_product(
         self,
         other: VectorField,
@@ -358,75 +250,91 @@ class VectorField(DataFieldBase):
             specify the output array to which the result is written. Note that the
             returned function is jitted with numba for speed.
         """
-        dim = self.grid.dim
 
-        if backend == "numba":
-            # create the dot product using a numba compiled function
+        def outer(
+            a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+        ) -> np.ndarray:
+            """calculate the outer product using numpy"""
+            return np.einsum("i...,j...->ij...", a, b, out=out)
 
-            # create the inner function calculating the dot product
+        if backend == "numpy":
+            # return the bare dot operator without the numba-overloaded version
+            return outer
+
+        elif backend == "numba":
+            # overload `outer` with a numba-compiled version
+
+            dim = self.grid.dim
+            num_axes = self.grid.num_axes
+
+            def check_rank(arr: Union[nb.types.Type, nb.types.Optional]) -> None:
+                """determine rank of field with type `arr`"""
+                arr_typ = arr.type if isinstance(arr, nb.types.Optional) else arr
+                if not isinstance(arr_typ, (np.ndarray, nb.types.Array)):
+                    raise nb.errors.TypingError(
+                        f"Arguments must be array, not  {arr_typ.__class__}"
+                    )
+                assert arr_typ.ndim == 1 + num_axes
+
+            # create the inner function calculating the outer product
             @register_jitable
             def calc(a: np.ndarray, b: np.ndarray, out: np.ndarray) -> np.ndarray:
-                """calculate dot product between fields `a` and `b`"""
+                """calculate outer product between fields `a` and `b`"""
                 for i in range(0, dim):
                     for j in range(0, dim):
                         out[i, j, :] = a[i] * b[j]
                 return out
 
-            def outer(
+            @overload(outer, inline="always")
+            def outer_ol(
                 a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
             ) -> np.ndarray:
-                """wrapper deciding whether the underlying function is called
-                with or without `out`."""
-                if out is None:
-                    out = np.empty((len(a),) + b.shape, dtype=get_common_dtype(a, b))
-                return calc(a, b, out)  # type: ignore
+                """numba implementation to calculate outer product between two fields"""
+                # get (and check) rank of the input arrays
+                check_rank(a)
+                check_rank(b)
+                in_shape = (dim,) + self.grid.shape
+                out_shape = (dim, dim) + self.grid.shape
 
-            @overload(outer)
-            def ol_outer(
-                a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
-            ) -> np.ndarray:
-                """wrapper deciding whether the underlying function is called
-                with or without `out`."""
-                if isinstance(a, nb.types.Number):
-                    # simple scalar call -> do not need to allocate anything
-                    raise RuntimeError("Dot needs to be called with fields")
-
-                elif isinstance(out, (nb.types.NoneType, nb.types.Omitted)):
-                    # function is called without `out`
+                if isinstance(out, (nb.types.NoneType, nb.types.Omitted)):
+                    # function is called without `out` -> allocate memory
                     dtype = get_common_numba_dtype(a, b)
 
-                    def f_with_allocated_out(
-                        a: np.ndarray, b: np.ndarray, out: np.ndarray
+                    def outer_impl(
+                        a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
                     ) -> np.ndarray:
                         """helper function allocating output array"""
-                        out = np.empty((len(a),) + b.shape, dtype=dtype)
-                        return calc(a, b, out=out)  # type: ignore
-
-                    return f_with_allocated_out  # type: ignore
+                        assert a.shape == b.shape == in_shape
+                        out = np.empty(out_shape, dtype=dtype)
+                        calc(a, b, out)
+                        return out
 
                 else:
-                    # function is called with `out` argument
-                    return calc  # type: ignore
+                    # function is called with `out` argument -> reuse `out` array
 
-        elif backend == "numpy":
-            # create the dot product using basic numpy functions
+                    def outer_impl(
+                        a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
+                    ) -> np.ndarray:
+                        """helper function without allocating output array"""
+                        # check input
+                        assert a.shape == b.shape == in_shape
+                        assert out.shape == out_shape  # type: ignore
+                        calc(a, b, out)
+                        return out  # type: ignore
 
-            def outer(
+                return outer_impl  # type: ignore
+
+            @jit
+            def outer_compiled(
                 a: np.ndarray, b: np.ndarray, out: Optional[np.ndarray] = None
             ) -> np.ndarray:
-                """calculates the outer product between two vector fields"""
-                if out is None:
-                    # TODO: Remove this construct once we make numpy 1.20 a minimal
-                    # requirement. Earlier version of numpy do not support out=None
-                    # correctly and we thus had to use this work-around
-                    return np.einsum("i...,j...->ij...", a, b)  # type: ignore
-                else:
-                    return np.einsum("i...,j...->ij...", a, b, out=out)
+                """numba implementation to calculate outer product between two fields"""
+                return outer(a, b, out)
+
+            return outer_compiled  # type: ignore
 
         else:
-            raise ValueError(f"Undefined backend `{backend}")
-
-        return outer
+            raise ValueError(f"Unsupported backend `{backend}")
 
     @fill_in_docstring
     def divergence(
@@ -448,7 +356,7 @@ class VectorField(DataFieldBase):
         Returns:
             :class:`~pde.fields.scalar.ScalarField`: result of applying the operator
         """
-        return self._apply_operator("divergence", bc=bc, out=out, **kwargs)  # type: ignore
+        return self.apply_operator("divergence", bc=bc, out=out, **kwargs)  # type: ignore
 
     @fill_in_docstring
     def gradient(
@@ -478,7 +386,7 @@ class VectorField(DataFieldBase):
         Returns:
             :class:`~pde.fields.tensorial.Tensor2Field`: result of applying the operator
         """
-        return self._apply_operator("vector_gradient", bc=bc, out=out, **kwargs)  # type: ignore
+        return self.apply_operator("vector_gradient", bc=bc, out=out, **kwargs)  # type: ignore
 
     @fill_in_docstring
     def laplace(
@@ -508,7 +416,7 @@ class VectorField(DataFieldBase):
         Returns:
             :class:`~pde.fields.vectorial.VectorField`: result of applying the operator
         """
-        return self._apply_operator("vector_laplace", bc=bc, out=out, **kwargs)  # type: ignore
+        return self.apply_operator("vector_laplace", bc=bc, out=out, **kwargs)  # type: ignore
 
     @property
     def integral(self) -> np.ndarray:
