@@ -31,12 +31,13 @@ from typing import (
     Union,
 )
 
+import numba as nb
 import numpy as np
 from numba.extending import is_jitted, overload, register_jitable
 
 from ..tools.cache import cached_method, cached_property
 from ..tools.docstrings import fill_in_docstring
-from ..tools.misc import Number, classproperty
+from ..tools.misc import Number, hybridmethod
 from ..tools.numba import jit
 from ..tools.typing import (
     CellVolume,
@@ -153,13 +154,12 @@ class GridBase(metaclass=ABCMeta):
     """list: axes that not described explicitly"""
     num_axes: int
     """int: Number of axes that are *not* assumed symmetrically"""
-    periodic: List[bool]
-    """list: Flags that describe which axes are periodic"""
 
     # mandatory, immutable, private attributes
     _axes_bounds: Tuple[Tuple[float, float], ...]
     _axes_coords: Tuple[np.ndarray, ...]
     _discretization: np.ndarray
+    _periodic: List[bool]
     _shape: Tuple[int, ...]
 
     # to help sphinx, we here list docstrings for classproperties
@@ -207,6 +207,11 @@ class GridBase(metaclass=ABCMeta):
     ) -> GridBase:
         raise NotImplementedError
 
+    @property
+    def periodic(self) -> List[bool]:
+        """list: Flags that describe which axes are periodic"""
+        return self._periodic
+
     @property
     def axes_bounds(self) -> Tuple[Tuple[float, float], ...]:
         """tuple: lower and upper bounds of each axis"""
@@ -415,19 +420,25 @@ class GridBase(metaclass=ABCMeta):
         return all(np.asarray(vols).ndim == 0 for vols in self.cell_volume_data)
 
     def difference_vector_real(self, p1: np.ndarray, p2: np.ndarray) -> np.ndarray:
-        """return the vector pointing from p1 to p2
+        """return vector(s) pointing from p1 to p2
 
         In case of periodic boundary conditions, the shortest vector is returned.
 
+        Warning:
+            This function assumes a Cartesian coordinate system and might thus not work
+            for curvilinear coordinates.
+
         Args:
             p1 (:class:`~numpy.ndarray`): First point(s)
             p2 (:class:`~numpy.ndarray`): Second point(s)
 
         Returns:
-            :class:`~numpy.ndarray`: The difference vectors between the points
-            with periodic  boundary conditions applied.
+            :class:`~numpy.ndarray`: The difference vectors between the points with
+            periodic  boundary conditions applied.
         """
         diff = np.atleast_1d(p2) - np.atleast_1d(p1)
+        assert diff.shape[-1] == self.num_axes
+
         for i, periodic in enumerate(self.periodic):
             if periodic:
                 size = self.axes_bounds[i][1] - self.axes_bounds[i][0]
@@ -439,6 +450,10 @@ class GridBase(metaclass=ABCMeta):
 
         This takes periodic boundary conditions into account if necessary.
 
+        Warning:
+            This function calculates the Euclidean distance and might thus give wrong
+            results for coordinates given in curvilinear coordinate systems.
+
         Args:
             p1 (:class:`~numpy.ndarray`): First position
             p2 (:class:`~numpy.ndarray`): Second position
@@ -813,13 +828,31 @@ class GridBase(metaclass=ABCMeta):
             # method is used directly
             register_operator(factory_func)
 
-    @classproperty  # type: ignore
+    @hybridmethod  # type: ignore
+    @property
     def operators(cls) -> Set[str]:  # @NoSelf
         """set: all operators defined for this class"""
         result = set()
+        # add all customly defined operators
         classes = inspect.getmro(cls)[:-1]  # type: ignore
         for anycls in classes:
             result |= set(anycls._operators.keys())  # type: ignore
+        if hasattr(cls, "axes"):
+            for ax in cls.axes:
+                result |= {f"d_d{ax}", f"d2_d{ax}2"}
+        return result
+
+    @operators.instancemethod
+    @property
+    def operators(self) -> Set[str]:
+        """set: all operators defined for this instance"""
+        # get all operators registered on the class
+        result = self.__class__.operators
+        if not hasattr(self.__class__, "axes"):
+            # add operators calculating derivate along a coordinate for the case where
+            # the axes argument is only defined on instances
+            for ax in self.axes:
+                result |= {f"d_d{ax}", f"d2_d{ax}2"}
         return result
 
     def _get_operator_info(self, operator: Union[str, OperatorInfo]) -> OperatorInfo:
@@ -942,64 +975,108 @@ class GridBase(metaclass=ABCMeta):
         bcs = self.get_boundary_conditions(bc, rank=operator.rank_in)
 
         # calculate shapes of the full data
+        shape_in_valid = (self.dim,) * operator.rank_in + self.shape
         shape_in_full = (self.dim,) * operator.rank_in + self._shape_full
         shape_out = (self.dim,) * operator.rank_out + self.shape
 
-        if backend == "numba":
-            # create a compiled function to apply the operator
+        # define numpy version of the operator
+        def apply_op(
+            arr: np.ndarray, out: Optional[np.ndarray] = None, args=None
+        ) -> np.ndarray:
+            """set boundary conditions and apply operator"""
+            assert arr.shape == shape_in_valid
+            # ensure `out` array is allocated
+            if out is None:
+                out = np.empty(shape_out, dtype=arr.dtype)
+            else:
+                assert out.shape == shape_out
+
+            # prepare input with boundary conditions
+            arr_full = np.empty(shape_in_full, dtype=arr.dtype)
+            arr_full[(...,) + self._idx_valid] = arr
+            bcs.set_ghost_cells(arr_full, args=args)
+
+            # apply operator
+            operator_raw(arr_full, out)
+
+            # return valid part of the output
+            return out
+
+        if backend in {"numpy", "scipy"}:
+            # return the bare operator without the numba-overloaded version
+            return apply_op
+
+        elif backend == "numba":
+            # overload `apply_op` with numba-compiled version
             set_ghost_cells = bcs.make_ghost_cell_setter()
             set_valid = self._make_set_valid()
 
             if not is_jitted(operator_raw):
                 operator_raw = jit(operator_raw)
 
-            @jit
-            def apply_op(
+            @overload(apply_op, inline="always")
+            def apply_op_ol(
                 arr: np.ndarray, out: Optional[np.ndarray] = None, args=None
             ) -> np.ndarray:
-                """applies operator to the data"""
-                if out is None:
-                    out = np.empty(shape_out, dtype=arr.dtype)
-                # prepare input with boundary conditions
-                arr_full = np.empty(shape_in_full, dtype=arr.dtype)
-                set_valid(arr_full, arr)
-                set_ghost_cells(arr_full, args=args)
+                """make numba implementation of the operator"""
+                if isinstance(out, (nb.types.NoneType, nb.types.Omitted)):
+                    # need to allocate memory for `out`
 
-                # apply operator
-                operator_raw(arr_full, out)
+                    def apply_op_impl(
+                        arr: np.ndarray, out: Optional[np.ndarray] = None, args=None
+                    ) -> np.ndarray:
+                        """allocates `out` and applies operator to the data"""
+                        assert arr.shape == shape_in_valid
 
-                # return valid part of the output
-                return out
+                        out = np.empty(shape_out, dtype=arr.dtype)
+                        # prepare input with boundary conditions
+                        arr_full = np.empty(shape_in_full, dtype=arr.dtype)
+                        set_valid(arr_full, arr)
+                        set_ghost_cells(arr_full, args=args)
 
-        elif backend in {"numpy", "scipy"}:
-            # create a numpy/scipy function to apply the operator
+                        # apply operator
+                        operator_raw(arr_full, out)
 
-            def apply_op(
+                        # return valid part of the output
+                        return out
+
+                else:
+                    # reuse provided `out` array
+
+                    def apply_op_impl(
+                        arr: np.ndarray, out: Optional[np.ndarray] = None, args=None
+                    ) -> np.ndarray:
+                        """applies operator to the data wihtout allocating out"""
+                        assert arr.shape == shape_in_valid
+                        assert out.shape == shape_out  # type: ignore
+
+                        # prepare input with boundary conditions
+                        arr_full = np.empty(shape_in_full, dtype=arr.dtype)
+                        set_valid(arr_full, arr)
+                        set_ghost_cells(arr_full, args=args)
+
+                        # apply operator
+                        operator_raw(arr_full, out)  # type: ignore
+
+                        # return valid part of the output
+                        return out  # type: ignore
+
+                return apply_op_impl  # type: ignore
+
+            @jit
+            def apply_op_compiled(
                 arr: np.ndarray, out: Optional[np.ndarray] = None, args=None
             ) -> np.ndarray:
                 """set boundary conditions and apply operator"""
-                # ensure result array is allocated
-                if out is None:
-                    out = np.empty(shape_out, dtype=arr.dtype)
-                else:
-                    assert out.shape == shape_out
+                return apply_op(arr, out, args)
 
-                # prepare input with boundary conditions
-                arr_full = np.empty(shape_in_full, dtype=arr.dtype)
-                arr_full[(...,) + self._idx_valid] = arr
-                bcs.set_ghost_cells(arr_full, args=args)
-
-                # apply operator
-                operator_raw(arr_full, out)
-
-                # return valid part of the output
-                return out
+            # return the compiled versions of the operator
+            return apply_op_compiled  # type: ignore
 
         else:
+            # simply return the operator if the backend was `numba` or `scipy`
             raise NotImplementedError(f"Undefined backend '{backend}'")
 
-        return apply_op  # type: ignore
-
     def slice(self, indices: Sequence[int]) -> GridBase:
         """return a subgrid of only the specified axes"""
         raise NotImplementedError(
