@@ -107,8 +107,14 @@ def add_scaled_colorbar(
     cbar = ax.figure.colorbar(axes_image, cax=cax, **kwargs)
 
     # disable the offset that matplotlib sometimes shows
-    cax.get_xaxis().get_major_formatter().set_useOffset(False)
-    cax.get_yaxis().get_major_formatter().set_useOffset(False)
+    try:
+        cax.get_xaxis().get_major_formatter().set_useOffset(False)
+    except AttributeError:
+        pass  # can happen for logarithmically formatted axes
+    try:
+        cax.get_yaxis().get_major_formatter().set_useOffset(False)
+    except AttributeError:
+        pass  # can happen for logarithmically formatted axes
 
     if label:
         cbar.set_label(label)
@@ -590,7 +596,7 @@ class PlottingContextBase:
         if self.fig is not None:
             import matplotlib.pyplot as plt
 
-            plt.figure(self.fig.number)
+            self.fig = plt.figure(self.fig.number)
 
     def __exit__(self, *exc):
         if self.initial_plot or not self.supports_update:
